@@ -77,7 +77,7 @@ if TYPE_CHECKING:
         # the server side and WebSocketClientConnection on the client
         # side.
         def on_ws_connection_close(
-            self, close_code: int = None, close_reason: str = None
+            self, close_code: Optional[int] = None, close_reason: Optional[str] = None
         ) -> None:
             pass
 
@@ -122,10 +122,10 @@ class _DecompressTooLargeError(Exception):
 class _WebSocketParams(object):
     def __init__(
         self,
-        ping_interval: float = None,
-        ping_timeout: float = None,
+        ping_interval: Optional[float] = None,
+        ping_timeout: Optional[float] = None,
         max_message_size: int = _default_max_message_size,
-        compression_options: Dict[str, Any] = None,
+        compression_options: Optional[Dict[str, Any]] = None,
     ) -> None:
         self.ping_interval = ping_interval
         self.ping_timeout = ping_timeout
@@ -190,7 +190,7 @@ class WebSocketHandler(tornado.web.RequestHandler):
 
     Web browsers allow any site to open a websocket connection to any other,
     instead of using the same-origin policy that governs other network
-    access from javascript.  This can be surprising and is a potential
+    access from JavaScript.  This can be surprising and is a potential
     security hole, so since Tornado 4.0 `WebSocketHandler` requires
     applications that wish to receive cross-origin websockets to opt in
     by overriding the `~WebSocketHandler.check_origin` method (see that
@@ -221,7 +221,7 @@ class WebSocketHandler(tornado.web.RequestHandler):
         request: httputil.HTTPServerRequest,
         **kwargs: Any
     ) -> None:
-        super(WebSocketHandler, self).__init__(application, request, **kwargs)
+        super().__init__(application, request, **kwargs)
         self.ws_connection = None  # type: Optional[WebSocketProtocol]
         self.close_code = None  # type: Optional[int]
         self.close_reason = None  # type: Optional[str]
@@ -280,8 +280,6 @@ class WebSocketHandler(tornado.web.RequestHandler):
             self.set_status(426, "Upgrade Required")
             self.set_header("Sec-WebSocket-Version", "7, 8, 13")
 
-    stream = None
-
     @property
     def ping_interval(self) -> Optional[float]:
         """The interval for websocket keep-alive pings.
@@ -468,7 +466,7 @@ class WebSocketHandler(tornado.web.RequestHandler):
         """
         pass
 
-    def close(self, code: int = None, reason: str = None) -> None:
+    def close(self, code: Optional[int] = None, reason: Optional[str] = None) -> None:
         """Closes this Web Socket.
 
         Once the close handshake is successful the socket will be closed.
@@ -571,7 +569,7 @@ class WebSocketHandler(tornado.web.RequestHandler):
             self._break_cycles()
 
     def on_ws_connection_close(
-        self, close_code: int = None, close_reason: str = None
+        self, close_code: Optional[int] = None, close_reason: Optional[str] = None
     ) -> None:
         self.close_code = close_code
         self.close_reason = close_reason
@@ -584,11 +582,11 @@ class WebSocketHandler(tornado.web.RequestHandler):
         # connection (if it was established in the first place,
         # indicated by status code 101).
         if self.get_status() != 101 or self._on_close_called:
-            super(WebSocketHandler, self)._break_cycles()
+            super()._break_cycles()
 
     def send_error(self, *args: Any, **kwargs: Any) -> None:
         if self.stream is None:
-            super(WebSocketHandler, self).send_error(*args, **kwargs)
+            super().send_error(*args, **kwargs)
         else:
             # If we get an uncaught exception during the handshake,
             # we have no choice but to abruptly close the connection.
@@ -670,7 +668,7 @@ class WebSocketProtocol(abc.ABC):
         self.close()  # let the subclass cleanup
 
     @abc.abstractmethod
-    def close(self, code: int = None, reason: str = None) -> None:
+    def close(self, code: Optional[int] = None, reason: Optional[str] = None) -> None:
         raise NotImplementedError()
 
     @abc.abstractmethod
@@ -724,7 +722,7 @@ class _PerMessageDeflateCompressor(object):
         self,
         persistent: bool,
         max_wbits: Optional[int],
-        compression_options: Dict[str, Any] = None,
+        compression_options: Optional[Dict[str, Any]] = None,
     ) -> None:
         if max_wbits is None:
             max_wbits = zlib.MAX_WBITS
@@ -773,7 +771,7 @@ class _PerMessageDeflateDecompressor(object):
         persistent: bool,
         max_wbits: Optional[int],
         max_message_size: int,
-        compression_options: Dict[str, Any] = None,
+        compression_options: Optional[Dict[str, Any]] = None,
     ) -> None:
         self._max_message_size = max_message_size
         if max_wbits is None:
@@ -996,7 +994,7 @@ class WebSocketProtocol13(WebSocketProtocol):
         self,
         side: str,
         agreed_parameters: Dict[str, Any],
-        compression_options: Dict[str, Any] = None,
+        compression_options: Optional[Dict[str, Any]] = None,
     ) -> Dict[str, Any]:
         """Converts a websocket agreed_parameters set to keyword arguments
         for our compressor objects.
@@ -1016,7 +1014,7 @@ class WebSocketProtocol13(WebSocketProtocol):
         self,
         side: str,
         agreed_parameters: Dict[str, Any],
-        compression_options: Dict[str, Any] = None,
+        compression_options: Optional[Dict[str, Any]] = None,
     ) -> None:
         # TODO: handle invalid parameters gracefully
         allowed_keys = set(
@@ -1259,7 +1257,7 @@ class WebSocketProtocol13(WebSocketProtocol):
             self._abort()
         return None
 
-    def close(self, code: int = None, reason: str = None) -> None:
+    def close(self, code: Optional[int] = None, reason: Optional[str] = None) -> None:
         """Closes the WebSocket connection."""
         if not self.server_terminated:
             if not self.stream.closed():
@@ -1287,6 +1285,9 @@ class WebSocketProtocol13(WebSocketProtocol):
             self._waiting = self.stream.io_loop.add_timeout(
                 self.stream.io_loop.time() + 5, self._abort
             )
+        if self.ping_callback:
+            self.ping_callback.stop()
+            self.ping_callback = None
 
     def is_closing(self) -> bool:
         """Return ``True`` if this connection is closing.
@@ -1365,10 +1366,10 @@ class WebSocketClientConnection(simple_httpclient._HTTPConnection):
     def __init__(
         self,
         request: httpclient.HTTPRequest,
-        on_message_callback: Callable[[Union[None, str, bytes]], None] = None,
-        compression_options: Dict[str, Any] = None,
-        ping_interval: float = None,
-        ping_timeout: float = None,
+        on_message_callback: Optional[Callable[[Union[None, str, bytes]], None]] = None,
+        compression_options: Optional[Dict[str, Any]] = None,
+        ping_interval: Optional[float] = None,
+        ping_timeout: Optional[float] = None,
         max_message_size: int = _default_max_message_size,
         subprotocols: Optional[List[str]] = [],
     ) -> None:
@@ -1408,8 +1409,11 @@ class WebSocketClientConnection(simple_httpclient._HTTPConnection):
                 "Sec-WebSocket-Extensions"
             ] = "permessage-deflate; client_max_window_bits"
 
+        # Websocket connection is currently unable to follow redirects
+        request.follow_redirects = False
+
         self.tcp_client = TCPClient()
-        super(WebSocketClientConnection, self).__init__(
+        super().__init__(
             None,
             request,
             lambda: None,
@@ -1420,7 +1424,7 @@ class WebSocketClientConnection(simple_httpclient._HTTPConnection):
             104857600,
         )
 
-    def close(self, code: int = None, reason: str = None) -> None:
+    def close(self, code: Optional[int] = None, reason: Optional[str] = None) -> None:
         """Closes the websocket connection.
 
         ``code`` and ``reason`` are documented under
@@ -1441,10 +1445,10 @@ class WebSocketClientConnection(simple_httpclient._HTTPConnection):
             self.connect_future.set_exception(StreamClosedError())
         self._on_message(None)
         self.tcp_client.close()
-        super(WebSocketClientConnection, self).on_connection_close()
+        super().on_connection_close()
 
     def on_ws_connection_close(
-        self, close_code: int = None, close_reason: str = None
+        self, close_code: Optional[int] = None, close_reason: Optional[str] = None
     ) -> None:
         self.close_code = close_code
         self.close_reason = close_reason
@@ -1466,9 +1470,7 @@ class WebSocketClientConnection(simple_httpclient._HTTPConnection):
     ) -> None:
         assert isinstance(start_line, httputil.ResponseStartLine)
         if start_line.code != 101:
-            await super(WebSocketClientConnection, self).headers_received(
-                start_line, headers
-            )
+            await super().headers_received(start_line, headers)
             return
 
         if self._timeout is not None:
@@ -1506,7 +1508,8 @@ class WebSocketClientConnection(simple_httpclient._HTTPConnection):
         return self.protocol.write_message(message, binary=binary)
 
     def read_message(
-        self, callback: Callable[["Future[Union[None, str, bytes]]"], None] = None
+        self,
+        callback: Optional[Callable[["Future[Union[None, str, bytes]]"], None]] = None,
     ) -> Awaitable[Union[None, str, bytes]]:
         """Reads a message from the WebSocket server.
 
@@ -1585,14 +1588,14 @@ class WebSocketClientConnection(simple_httpclient._HTTPConnection):
 
 def websocket_connect(
     url: Union[str, httpclient.HTTPRequest],
-    callback: Callable[["Future[WebSocketClientConnection]"], None] = None,
-    connect_timeout: float = None,
-    on_message_callback: Callable[[Union[None, str, bytes]], None] = None,
-    compression_options: Dict[str, Any] = None,
-    ping_interval: float = None,
-    ping_timeout: float = None,
+    callback: Optional[Callable[["Future[WebSocketClientConnection]"], None]] = None,
+    connect_timeout: Optional[float] = None,
+    on_message_callback: Optional[Callable[[Union[None, str, bytes]], None]] = None,
+    compression_options: Optional[Dict[str, Any]] = None,
+    ping_interval: Optional[float] = None,
+    ping_timeout: Optional[float] = None,
     max_message_size: int = _default_max_message_size,
-    subprotocols: List[str] = None,
+    subprotocols: Optional[List[str]] = None,
 ) -> "Awaitable[WebSocketClientConnection]":
     """Client-side websocket support.
 
